/*****************************************************************************
*                                                                            *
* void DFT_1Dim(int Zeilen_Richtung, int Nummer)                             *
*                                                                            *
******************************************************************************
*                                                                            *
* Eindimensionale diskrete Fourier-Transformation (Zeilen oder Spalten)      *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 13.12.1992                                               *
* Letzte Aenderung: 21.04.1993                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von: ClearTextWindow(), WriteText(), Read_Int(), Read_Char(),       *
*             Show_Pic(), Histo_darst()                                      *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"


void DFT_1Dim(int Zeilen_Richtung, int Nummer)
{
  int BildNr1, BildNr2;           /* Quellbild und Zielbild (Spektren)      */
  int Punkt_Zahl;                 /* Anzahl der Punkte pro Zeile/Spalte     */
  long i,j;                       /* Adresse des Quellbildes                */
  long Index1;
  float Trafo_Werte[S_RES_MAX];   /* zu transformierende Zeile/Spalte       */
  float Amplitude[S_RES_MAX];     /* Werte des Amplitudenspektrums          */
  float Phase[S_RES_MAX];         /* Werte des Phasenspektrums              */
  float Real_Teil, Imag_Teil;
  float Phi, Phi_Inkr;            /* Winkel und Winkel-Inkrement            */

  ClearTextWindow(35,18,80,25);
  WriteText(35,18,"* DFT - eindimensional *");

  gotoxy(35,20);
  BildNr1=Read_Int("Quell-Bild ", 1, ':');
  /* Gueltige Bild-Nummer eingegeben? Ist dieses Bild ueberhaupt benutzt?   */
  if ((BildNr1<=0) || (BildNr1>MAX_PICTURE)) return;
  BildNr1--;                          /* die Zaehlung faengt immer bei 0 an */
  if (!Bild[BildNr1].Used) return;

  gotoxy(35,21);
  BildNr2=Read_Int("Ziel-Bild  ", 2, ':');
  /* Gueltige Bild-Nummer eingegeben?                                       */
  if ((BildNr2<=0) || (BildNr2>MAX_PICTURE)) return;
  BildNr2--;                          /* die Zaehlung faengt immer bei 0 an */

  Index1 = (long) (BildNr1) * PIC_SIZE;  /* Anfangsadresse des Quellbildes  */

  /* Zeile/Spalte in den Vektro Trafo_Werte[] umkopieren                    */
  if (Zeilen_Richtung) {
     Punkt_Zahl=Bild[BildNr1].Zeilen;
     while ((Nummer<0) || (Nummer>=Punkt_Zahl)) {
        gotoxy(35,22);
        Nummer=Read_Int("Zeile Nummer", INT_NO_DEFAULT, ':');
     }
     for (i=0; i<Punkt_Zahl; i++)
         Trafo_Werte[i]=(float)Pictures[Index1 +
                              (long)(Nummer)*(long)(Bild[BildNr1].Spalten)+i];
  }
  else {
     Punkt_Zahl=Bild[BildNr1].Spalten;
     while ((Nummer<0) || (Nummer>=Punkt_Zahl)) {
        gotoxy(35,22);
        Nummer=Read_Int("Spalte Nummer", INT_NO_DEFAULT, ':');
     }
     for (i=0; i<Punkt_Zahl; i++)
         Trafo_Werte[i]=(float)Pictures[Index1 +
                              (long)(Nummer)+(long)(Bild[BildNr1].Spalten)*i];
  }

  /* Zuerst den Zeilen-Histogramm-Verlauf anzeigen                          */
  WriteText(35,24,"Zeilen-Histogramm");
  Histo_darst(BildNr2, Punkt_Zahl, Trafo_Werte, 0);
  Show_Pic(BildNr2, BildNr2+1);

  gotoxy(35,25);
  Read_Char("Weiter mit <RETURN>", '<');
  ClearTextWindow(35,24,80,25);

  /* Jetzt die diskrete Fourier-Transformation genau nach Formel berechnen  */
  for (i=0; i<Punkt_Zahl; i++) {
      Real_Teil=0;
      Imag_Teil=0;
      Phi      =0;
      Phi_Inkr = 2.0*PI*i/(float)(Punkt_Zahl);
      for (j=0; j<Punkt_Zahl; j++) {
          Real_Teil +=Trafo_Werte[j] * cos(Phi);
          Imag_Teil +=Trafo_Werte[j] * sin(Phi);
          Phi -=Phi_Inkr;
      }
      Real_Teil /=Punkt_Zahl;             /* Normierung, ggf. ueberfluessig */
      Imag_Teil /=Punkt_Zahl;
      /* Amplitudenspektrum und Phasenspektrum in den Vektoren ablegen      */
      Amplitude[i] = sqrt(Real_Teil*Real_Teil + Imag_Teil*Imag_Teil);
      Phase[i]     = atan2(Real_Teil, Imag_Teil);
  }

  WriteText(35,24,"DFT-Amplitude    ");
  Amplitude[0] /=4.0;        /* DC-Anteil fuer Gesamtskalierung verkleinern */
  Histo_darst(BildNr2, Punkt_Zahl, Amplitude, S_RES_MAX/2);
  Show_Pic(BildNr2, BildNr2+1);

  gotoxy(35,25);
  Read_Char("Weiter mit <RETURN>", '<');
  ClearTextWindow(35,24,80,25);

  WriteText(35,24,"DFT-Phase        ");
  Histo_darst(BildNr2, Punkt_Zahl, Phase, 0);
  Show_Pic(BildNr2, BildNr2+1);

}
