/*****************************************************************************
*                                                                            *
* void      FFT_1DIM(int Vorwaerts_Trafo, int Anzahl, int ld_Anzahl,         *
*                     float Dat_Real[], float Dat_Imag[])                    *
*                                                                            *
******************************************************************************
*                                                                            *
* Eindimensionale Fast-Fourier-Transformation (Zeilen oder Spalten). Die     *
* Werte werden schon entsprechend sortiert uebergeben.                       *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 13.12.1992                                               *
* Letzte Aenderung: 16.12.1992                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Input:  Vorwaerts_Trafo = Flag ob Vorwaerts- oder Rueckwaertstransformat.  *
*         Anzahl          = Anzahl der Stuetzsstellen                        *
*         ld_Anzahl       = ld(Anzahl)                                       *
*         Dat_Real[]      = Vektor mit den Real-Werten                       *
*         Dat_Imag[]      = Vektor mit den Imaginaer-Werten                  *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von:                                                                *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"


void FFT_1Dim(int Vorwaerts_Trafo, int Anzahl, int ld_Anzahl, float Dat_Real[], float Dat_Imag[])
{
  double Phi, Phi_Inkr;            /* Winkel und Inkrement                  */
  int i,j,k,l;
  int m, Haelfte;
  float W_Imag, W_Real;            /* Imaginaer- und Real-Wert der Wurzel W */
  float Tmp_Real, Tmp_Imag;

  /* iteratives Vorgehen bei der FFT-Berechnung, jedoch genau nach Formel!  */
  for (i=0; i<ld_Anzahl; i++) {
      m = 2;
      m <<= i;
      Haelfte = m >> 1;
      Phi = 0.0;
      Phi_Inkr = 2.0 * PI / (double)(m);
      for (j=0; j<Haelfte; j++) {
          W_Real = (float)cos(Phi);
          W_Imag = (float)sin(Phi);
          if (Vorwaerts_Trafo) W_Imag = -W_Imag;
          Phi -= Phi_Inkr;
          k=j;
          do {
             l=k+Haelfte;
             Tmp_Real = W_Real * Dat_Real[l] - W_Imag * Dat_Imag[l];
             Tmp_Imag = W_Real * Dat_Imag[l] + W_Imag * Dat_Real[l];
             Dat_Real[l] = Dat_Real[k] - Tmp_Real;
             Dat_Imag[l] = Dat_Imag[k] - Tmp_Imag;
             Dat_Real[k] += Tmp_Real;
             Dat_Imag[k] += Tmp_Imag;
             k += m;
          } while ( k < Anzahl );
      }
  }

  if (Vorwaerts_Trafo) {  /* bei der Vorwaertstransformation Normierung 1/N */
     for (j=0; j<Anzahl; j++) {
         Dat_Real[j] /=(float)Anzahl;
         Dat_Imag[j] /=(float)Anzahl;
     }
  }

}

