/*****************************************************************************
*                                                                            *
* void      Histogramm(int BildNr1, int BildNr2)                             *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellt ein Histogramm von BildNr1 und schreibt das Ergebnis in BildNr2   *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 02.10.1992                                               *
* Letzte Aenderung: 24.10.1992                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Input: BildNr1 = Nummer des Ausgangsbildes (1<=BildNr1<=MAX_PICTURE).      *
*        BildNr2 = Nummer des Histogrammbildes (1<=BildNr2<=MAX_PICTURE).    *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von: ClearTextWindow(), WriteText(), Read_Int(),                    *
*             Make_Icon(), Show_Pic()                                        *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"

#define HIGH  250                  /* Maximale Hoehe eine Histogramm-Saeule */


void Histogramm(int BildNr1, int BildNr2)
{
  unsigned int  Grau[MAX_COLOR];   /* Haeufigkeit der einzelnen Grauwerte   */
  unsigned int  Max_Haeufigkeit, i;
  float         Skalier_Faktor;
  long z, s;                       /* Zeilen- und Spaltenzaehler            */
  long Index1, Index2;             /* Anfangsadresse von Bild 1 und Bild 2  */

  /* Titel nur anzeigen, falls keine bzw. falsche Parameter angegeben       */
  if ((BildNr1==FALSE) || (BildNr2==FALSE)) {
     ClearTextWindow(35,18,80,25);
     WriteText(35,18,"* Histogramm erstellen *");
  }

  if (BildNr1 == FALSE) {
     gotoxy(35,20);
     BildNr1=Read_Int("Quell-Bild      ", 1, ':');
  }
  /* Gueltige Bild-Nummer eingegeben? Ist dieses Bild ueberhaupt benutzt?   */
  if ((BildNr1<=0) || (BildNr1>MAX_PICTURE)) return;
  BildNr1--;                          /* die Zaehlung faengt immer bei 0 an */
  if (!Bild[BildNr1].Used) return;

  if (BildNr2 == FALSE) {
     gotoxy(35,21);
     BildNr2=Read_Int("Histogramm-Bild ", 2, ':');
  }
  /* Gueltige Bild-Nummer eingegeben?                                       */
  if ((BildNr2<=0) || (BildNr2>MAX_PICTURE)) return;
  BildNr2--;                          /* die Zaehlung faengt immer bei 0 an */

  Index1 = (long) (BildNr1) * PIC_SIZE;  /* Anfangsadressen der beiden ...  */
  Index2 = (long) (BildNr2) * PIC_SIZE;  /* ... Bilder.                     */

  /* Initialisierung des Ziel-Histogrammbildes und der Haeufigkeitszaehler  */
  for (z=0; z<PIC_SIZE; z++) Pictures[Index2+z]=SCHWARZ;
  for (i=0; i<MAX_COLOR; i++) Grau[i]=0;

  for (z=0; z<Bild[BildNr1].Zeilen; z++)
      for (s=0; s<Bild[BildNr1].Spalten; s++)
          Grau[Pictures[Index1++]]++;
  
  Max_Haeufigkeit=0;       /* Groesste Haeufigkeit zur Normierung ermitteln */
  for (i=0; i<MAX_COLOR; i++)
      if (Grau[i] > Max_Haeufigkeit)
         Max_Haeufigkeit=Grau[i];

  /* Abbruch, falls Max_Haeufigkeit==0, d.h. kein Bild da                   */
  if (!Max_Haeufigkeit)  {
     ShowError("Histogramm kann nicht erstellt werden.");
     return;
  }

  Skalier_Faktor = (float)(HIGH) / (float)Max_Haeufigkeit;

  for (s=0; s<S_RES_MAX; s++) {
      Max_Haeufigkeit=(int)(Skalier_Faktor*(float)Grau[s]);
      if (Max_Haeufigkeit>HIGH) Max_Haeufigkeit=HIGH; /* evtl.Rundungsfehler*/
      /* Weisser Stab der richtigen Hoehe in das Bild einzeichnen           */
      for (z=HIGH; z>=(HIGH-Max_Haeufigkeit); z--)
          Pictures[Index2+z*S_RES_MAX+s] = WEISS;
  }

  /* Erzeugung eines Graukeils zur besseren Darstellung des Histogramms ... */
  for (z=251; z<Z_RES_MAX; z++)
      for (s=0; s<S_RES_MAX; s++)
          Pictures[Index2+z*S_RES_MAX+s] = s;

  /* ... und eine Markierung bei der Graustufe 128 setzen.                  */
  for (z=251; z<S_RES_MAX; z++)
      Pictures[Index2 + z*S_RES_MAX + S_RES_MAX/2]=SCHWARZ;

  /* Noch Bildinformationen fuer das Histogrammbild setzen ...              */
  Bild[BildNr2].Zeilen =Z_RES_MAX;
  Bild[BildNr2].Spalten=S_RES_MAX;
  Bild[BildNr2].Used   =TRUE;
  Bild[BildNr2].Color  =FALSE;

  /* ... und dann die Ergebnisse anzeigen.                                  */
  Make_Icon(BildNr2);
  Show_Pic(BildNr2, BildNr2+1);

}
