/*****************************************************************************
*                                                                            *
* void      Intens_Skalier(int BildNr)                                       *
*                                                                            *
******************************************************************************
*                                                                            *
* Fuehrt eine Grauwert-Intensitaetsskalierung nach O'Gorman durch.           *
* (vergleiche normale Histogrammerstellung)                                  *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 08.10.1992                                               *
* Letzte Aenderung: 23.10.1992                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Input:  BildNr = Nummer des zu bearbeitenden Bildes                        *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von: ClearTextWindow(), WriteText(), Read_Int(),                    *
*             Make_Icon(), Show_Pic()                                        *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"


void Intens_Skalier(int BildNr)
{
  unsigned char Grauwert[MAX_COLOR];/* transformierte Grauwerte             */
  unsigned int  Grau[MAX_COLOR];    /* Haeufigkleit der einzelnen Grauwerte */
  long          Grausum[MAX_COLOR]; /* kumulative Haeufigkeit der Grauwerte */
  long Index, Index1;               /* Adresse des Bildes                   */
  int          s, z, i;
  unsigned int X_Low, X_High;       /* kleinster/groesster vorkommender Grw.*/
  float        T;


  if (!BildNr) {
     ClearTextWindow(35,18,80,25);
     WriteText(35,18,"* Intensitaetsskalierung *");

     gotoxy(35,20);
     BildNr=Read_Int("Bild Nr. ", 1, ':');
  }
  /* Gueltige Bild-Nummer eingegeben? Falls nicht, dann Abbruch             */
  if ((BildNr<=0) || (BildNr>MAX_PICTURE)) return;
  BildNr--;                           /* die Zaehlung faengt immer bei 0 an */

  /* Initialisierung aller Arrays und Variablen                             */
  for (i=0; i<MAX_COLOR; Grau[i++]=0 );

  Index  = (long) (BildNr) * PIC_SIZE;  /* Adresse des Quell-/Zielbildes    */
  Index1 = Index;

  /* Haeufigkeit der einzelnen Grauwert ermitteln                           */
  for (z=0; z<Bild[BildNr].Zeilen; z++)
      for (s=0; s<Bild[BildNr].Spalten; s++)
          Grau[Pictures[Index1++]]++;

  /* Den kleinsten vorkommenden Grauwert X_Low suchen                       */
  i=(-1);
  while ( !Grau[++i] ); 
  X_Low=Grau[i];

  /* Den groessten vorkommenden Grauwert X_High suchen                      */
  i=MAX_COLOR;
  while ( !Grau[--i] ); 
  X_High=Grau[i];

  /* kummulatives Histogramm berechnen                                      */
  Grausum[0]=Grau[0];
  for (i=1; i<MAX_COLOR; i++) Grausum[i]=Grau[i]+Grausum[i-1];

  /* Transformationskennlinie T berechnen ...                               */
  T=((float)(Grausum[255]) - 0.5*((float)X_High+(float)X_Low));
  if (T != 0.0) T=(float)(WEISS)/T;

  /* ... und aus diesen Teilergebnissen die neuen Grauwerte ermitteln       */
  for (i=0; i<MAX_COLOR; i++)
      Grauwert[i]=(int)(T * ((float)Grausum[i] - 0.5*((float)Grau[i]+(float)X_Low)));

  /* Zur Vermeidung von Nebeneffekten bei Rundungsfehlern Randwerte setzen  */
  Grauwert[0]=0;
  Grauwert[255]=255;

  /* Zum Schluss die eigentliche Umsetzung der Grauwert vornehmen           */
  Index1=Index;
  for (s=0; s<Bild[BildNr].Spalten; s++)
      for (z=0; z<Bild[BildNr].Zeilen; z++)
          Pictures[Index1]=Grauwert[ Pictures[Index1++] ];

  Make_Icon(BildNr);
  Show_Pic(BildNr, BildNr+1);

}
