/*****************************************************************************
*                                                                            *
* void      Skalierung()                                                     *
*                                                                            *
******************************************************************************
*                                                                            *
* Vergroessert/verkleinert einen Ausschnitt aus einem Bild                   *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 09.10.1992                                               *
* Letzte Aenderung: 22.10.1992                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von: ClearTextWindow(), WriteText(), Read_Int(),                    *
*             Make_Icon(), Show_Pic()                                        *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"


void Skalierung()
{
  int   BildNr1, BildNr2;         /* Bildnummer von Quell- und Zielbild     */
  long  z, s, zz, ss;             /* Variablen fuer Zeilen und Spaltenindex */
  long  Index1, Index2;           /* Adresse der beiden Bilder              */
  int   SDest, ZDest;             /* Zeile, Spalte des Zielbildes           */
  int   SOffset, ZOffset;         /* Offset von Zeile und Spalte            */
  float S_Strich, Z_Strich;
  float SScale, ZScale;           /* Skalierungsfaktor fuer Zeile, Spalte   */
  int   Z_Res_Max, S_Res_Max;     /* Aufloesung des Quellbildes             */
  int   Z_Res_Neu, S_Res_Neu;     /* Aufloesung des Zielbildes              */


  ClearTextWindow(35,18,80,25);
  WriteText(35,18,"* Skalierung eines Ausschnittes *");

  gotoxy(35,20);
  BildNr1=Read_Int("Quell-Bild ", 1, ':');
  /* Gueltige Bild-Nummer eingegeben?                                       */
  if ((BildNr1<=0) || (BildNr1>MAX_PICTURE)) return;
  BildNr1--;                          /* die Zaehlung faengt immer bei 0 an */
  if (!Bild[BildNr1].Used)   return;

  gotoxy(35,21);
  BildNr2=Read_Int("Ziel-Bild  ", 2, ':');
  /* Gueltige Bild-Nummer eingegeben?                                       */
  if ((BildNr2<=0) || (BildNr2>MAX_PICTURE)) return;
  BildNr2--;                          /* die Zaehlung faengt immer bei 0 an */

  Index1 = (long) (BildNr1) * PIC_SIZE; /* Adresse der beiden Bilder        */
  Index2 = (long) (BildNr2) * PIC_SIZE;


  WriteText(35, 23, "Skalierungsfaktor (0=Abbruch):");
  gotoxy(35,24);
  ZScale = Read_Float("In Zeilen-Richtung  ", FLOAT_NO_DEFAULT, '?' );
  if (!ZScale) return;                                    /* ==> Ausstieg ! */

  gotoxy(35,25);
  SScale = Read_Float("In Spalten-Richtung ", FLOAT_NO_DEFAULT, '?' );
  if (!SScale) return;                                    /* ==> Ausstieg ! */


  ClearTextWindow(35,20,80,25);
  gotoxy(35,20);
  ZOffset = Read_Int( "Zeilen-Offset  (Quelle) ", 0, ':' );
  gotoxy(35,21);
  SOffset = Read_Int( "Spalten-Offset (Quelle) ", 0, ':' );
  gotoxy(35,22);
  ZDest   = Read_Int( "Zeilen-Offset  (Ziel)   ", 0, ':' );
  gotoxy(35,23);
  SDest   = Read_Int( "Spalten-Offset (Ziel)   ", 0, ':' );

  Z_Res_Max = Bild[BildNr1].Zeilen;
  S_Res_Max = Bild[BildNr1].Spalten;

  /* Neue Aufloesung berechnung und ggf. Groesse auf Maximum zuruecksetzen  */
  Z_Res_Neu = (int) ( ((float)(Z_Res_Max) - ZOffset) * ZScale + (float) ZDest );
  S_Res_Neu = (int) ( ((float)(S_Res_Max) - SOffset) * SScale + (float) SDest );
  if (Z_Res_Neu > Z_RES_MAX) Z_Res_Neu = Z_RES_MAX;
  if (S_Res_Neu > S_RES_MAX) S_Res_Neu = S_RES_MAX;

  /* Ziel-Bild komplett mit SCHWARZ loeschen                                */
  for (s=0; s<PIC_SIZE; s++) Pictures[Index2+s]=SCHWARZ;

  /* Formel:
     s' = s*sscale
     z' = s*zscale
     Zur Vermeidung von Luecken wird vom Zielbild rueckgerechnet            */
 
  S_Strich = 1.0/SScale;
  Z_Strich = 1.0/ZScale;

  /* Bei der Skalierung sind die eingegebenen Offsets zu beachten           */
  for (z=ZDest; z<Z_Res_Neu; z++) {
      zz = (int)( (z-ZDest) * Z_Strich +  ZOffset);
      for (s=SDest; s<S_Res_Neu; s++) {
          ss = (int) ( (s-SDest) * S_Strich + SOffset);
          /* nur falls der Bildpunkt im Bild liegt wird er uebernommen      */
          if ( (s>=0) && (s<S_RES_MAX) && (z>=0) && (z<Z_RES_MAX) )
             Pictures[Index2+z*(long)S_Res_Neu+s] =
                     Pictures[Index1+zz*(long)S_Res_Max+ss];
      }
  }

  /* Bildinformation zum Schluss korrekt setzen und ...                     */
  Bild[BildNr2].Zeilen =Z_Res_Neu;
  Bild[BildNr2].Spalten=S_Res_Neu;
  Bild[BildNr2].Used   =TRUE;
  Bild[BildNr2].Color  =FALSE;

  /* ... Bild anzeigen.                                                     */
  Make_Icon(BildNr2);
  Show_Pic(BildNr2, BildNr2+1);

}
