/*****************************************************************************
*                                                                            *
* void      Thinning_Ji(int BildNr)                                          *
*                                                                            *
******************************************************************************
*                                                                            *
* Skelettierung nach dem Artikel von Y. Ji - mit Ergaenzung fuer Endpunkte   *
* Proceedings 'Mustererkennung' 1989, Seite 72-76                            *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 25.04.1993                                               *
* Letzte Aenderung: 28.05.1993                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Input: BildNr  = Nummer des Ausgangsbildes (1<=BildNr1<=MAX_PICTURE).      *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von: ClearTextWindow(), WriteText(), Geduld(), Read_Int(),          *
*             Make_Icon(), Show_Pic()                                        *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"

#define SKELETT_PKT         1    /* Markierung der einzelnen Bildpunkte ... */
#define NICHT_SKELETT_PKT   2    /* ... zur einfacheren Berechnung.         */

#define PIXEL_ZAHL      20000    /* maximal PIXEL_ZAHL Kantenpunkte erlaubt */
#define PIXEL_ZAHL_2    40000

#define Buffer1(i, j) Buffer1[(long)(i)*2 + (long)(j)]
#define Buffer2(i, j) Buffer2[(long)(i)*2 + (long)(j)]


void Thinning_Ji(int BildNr)
{
  unsigned int k, l, m, kmax, lmax;   /* Zaehler fuer Kanten-/Skelettpunkte */
  int j;
  long z, s, a, b;                    /* Zeilen-/Spaltenindizes             */
  int Wert, Pkt_entfernt;

  unsigned char Richtung, Endpkt;
  unsigned char far *Liste;           /* dynamische allokierte Felder       */
  unsigned char far *Buffer1;
  unsigned char far *Buffer2;

  unsigned char Muster[7];
  long  Zmax, Smax, Index;


  /* Titel nur anzeigen, falls keine bzw. falsche Parameter angegeben       */
  if (BildNr==FALSE) {
     ClearTextWindow(35,18,80,25);
     WriteText(35,18,"* Skelettierung Ji (mod.) *");
  }

  if (BildNr == FALSE) {
     gotoxy(35,20);
     BildNr=Read_Int("Bild Nr. ", 1, ':');
  }
  /* Gueltige Bild-Nummer eingegeben? Ist dieses Bild ueberhaupt benutzt?   */
  if ((BildNr<=0) || (BildNr>MAX_PICTURE)) return;
  BildNr--;                           /* die Zaehlung faengt immer bei 0 an */
  if (!Bild[BildNr].Used) return;

  Geduld();

  Index = (long)BildNr*PIC_SIZE;      /* Informationen ueber aktuelles Bild */
  Zmax  = Bild[BildNr].Zeilen;
  Smax  = Bild[BildNr].Spalten;

  /* der Bildrand (1-Pixel) muss bei diesem Verfahren geloescht werden      */
  for (z=0; z<Zmax; z++) {
      Pictures[Index + z*Smax          ] = 0;
      Pictures[Index + z*Smax +(Smax-1)] = 0;
  }
  for (s=0; s<Smax; s++) {
      Pictures[Index +                 s] = 0;
      Pictures[Index + (Zmax-1)*Smax + s] = 0;
  }

  /* Alle Bildpunkte in 0/1 zur leichteren Berechnung umkodieren            */
  k=0;
  for (z=1; z<(Zmax-1); z++)
      for (s=1; s<(Smax-1); s++) {
          if (Pictures[Index + z*Smax +s]) k++;
          Pictures[Index + z*Smax + s]  =
                  Pictures[Index + z*Smax + s]==WEISS ? 1:0;
      }

  if (k>=PIXEL_ZAHL) {          /* falls zuviele Kantenpunkte, dann Abbruch */
     ShowError("Zuviele Kantenpunkte. Abbruch.");
     for (z=0; z<PIC_SIZE; z++)
         Pictures[Index + z] = (Pictures[Index + z] ? WEISS:SCHWARZ);
     return;                    /* ==> Abbruch                              */
  }

  /* Den fuer die weiteren Berechnungen benoetigten Speicher anfordern. Es  */
  /* muessten ca. 150KB frei sein. Benoetigt werden hier ca. 100KB.         */
  Liste     = farmalloc(PIXEL_ZAHL);
  Buffer1   = farmalloc(PIXEL_ZAHL_2);
  Buffer2   = farmalloc(PIXEL_ZAHL_2);

  if ((Liste==NULL) || (Buffer1==NULL) || (Buffer2==NULL)) {
     ShowError("Nicht genug Speicher frei. Abbruch");
     return;                    /* ==> Abbruch                              */
  }

  k = 0;
  l = 0;
  for (z=1; z<(Zmax-1); z++)
      for (s=1; s<(Smax-1); s++) {
          if (Pictures[Index + z*Smax + s]) {       /* ueberh. ein Pixel da?*/
             Wert = Code(Index, Smax, z, s, Muster);/* berechne Nachbar-Code*/
             if ((Wert < 255) && (Wert > 0)) {      /* Konturpunkt ?        */
                Buffer1(k, 0) = (unsigned char)z;   /* dann die Koordinaten */
                Buffer1(k, 1) = (unsigned char)s;   /* speichern.           */
                k++;
             }
             else if (Wert == 255) {                /* Innenpunkt ?         */
                Buffer2(l, 0) = (unsigned char)z;   /* dann die Koordinaten */
                Buffer2(l, 1) = (unsigned char)s;   /* gesondert speichern. */
                l++;
             }
          }
      }

  kmax = k;
  lmax = l;
  Pkt_entfernt = TRUE;      /* Flag, dass Punkte entfernt wurden oder nicht */

  while (Pkt_entfernt) {    /* solange noch Punkte entfertn wurden ...      */
        Pkt_entfernt = FALSE;
        for (k=0; k<PIXEL_ZAHL; k++) Liste[k]=0;

        for (j=0; j<=6; j+=2) {   /* j ist Zaehler fuer die Skelettierungsr.*/
            for (k=0; k<kmax; k++) {
                Liste[k]=0;       /* Liste der bearbeiteten Punkte          */
                a = Buffer1(k, 0);/* Koordinaten des bereits gefundenen ... */
                b = Buffer1(k, 1);/* ... Kantenpunktes holen.               */
                if (Pictures[Index + a*Smax + b]) {       /* Kantenpunkt ?  */
                   /* Bestimmung der Nachbarpunkte bei Endpunkten           */
                   /* Ein Pixel darf ebenfalls nicht nicht entfernt werden, */
                   /* falls er ein Endpunkt ist (Linienverkuerzung).        */
                   /* ----------------------------------------------------- */
                   /*        0 1 0       0 0 0       0 0 0       0 0 0      */
                   /*        0 p 0       0 p 1       0 p 0       1 p 0      */
                   /*        0 0 0       0 0 0       0 1 0       0 0 0      */
                   /* ----------------------------------------------------- */

                   Wert = Code(Index, Smax, a, b, Muster);
                   Endpkt = ((Wert==0x01) || (Wert==0x04) ||
                             (Wert==0x10) || (Wert==0x40) || (Wert==0x00));

                   /* Jetzt erst folgt der eigentliche Test auf Skelettpunkt*/
                   switch (j) {
                      case 0:
                          Richtung = (Pictures[Index + (a-1)*Smax + (b  )]==0);
                          if (Richtung) {         /* Skelettierung von oben */
                             if (Muster[1] || Muster[3] || Muster[4] || Muster[5] || Muster[6] || Endpkt)
                                Liste[k] = SKELETT_PKT;
                             else
                                Liste[k] = NICHT_SKELETT_PKT;
                          }
                          break;

                      case 2:
                          Richtung = (Pictures[Index + (a  )*Smax + (b+1)] == 0);
                          if (Richtung) {         /* Skelettierung von oben */
                             if (Muster[2] || Muster[3] || Muster[4] || Muster[5] || Muster[6] || Endpkt)
                                Liste[k] = SKELETT_PKT;
                             else
                                Liste[k] = NICHT_SKELETT_PKT;
                          }
                          break;

                      case 4:
                          Richtung = (Pictures[Index + (a+1)*Smax + (b  )] == 0);
                          if (Richtung) {         /* Skelettierung von oben */
                             if (Muster[1] || Muster[3] || Muster[4] || Muster[5] || Muster[6] || Endpkt)
                                Liste[k] = SKELETT_PKT;
                             else
                                Liste[k] = NICHT_SKELETT_PKT;
                          }
                          break;

                      case 6:
                          Richtung = (Pictures[Index + (a  )*Smax + (b-1)] == 0);
                          if (Richtung) {         /* Skelettierung von oben */
                             if (Muster[2] || Muster[3] || Muster[4] || Muster[5] || Muster[6] || Endpkt)
                                Liste[k] = SKELETT_PKT;
                             else
                                Liste[k] = NICHT_SKELETT_PKT;
                          }
                          break;
                   }
                }
            }

            for (k=0; k<kmax; k++)
                 /* Falls der Punkt kein Skelettpunkt war, wird er geloescht*/
                if (Liste[k] == NICHT_SKELETT_PKT) {
                   a = Buffer1(k, 0);
                   b = Buffer1(k, 1);
                   Pictures[Index + a*Smax + b] = SCHWARZ;
                   Pkt_entfernt = TRUE;/* Flag f. weiteren Durchlauf setzen */
                }
        }

        m = 0;
        for (k=0; k<kmax; k++)
            if (Liste[k] == 0) { /* Der Punkt wurde noch nicht untersucht,  */
                                 /* da er in den 4 Hauptachsen von '1'      */
                                 /* umgeben war.                            */
               Buffer1(m, 0) = Buffer1(k, 0);  /* Der Punkt wird jetzt aufg.*/
               Buffer1(m, 1) = Buffer1(k, 1);
               m++;
            }

        k = m-1;
        m = 0;
        for (l=0; l<lmax; l++) { /* Untersuchung der verbliebenen Innenpkte */
                                 /* auf ihre Nachbarschaft.                 */
            a = Buffer2(l, 0);
            b = Buffer2(l, 1);
            Wert = Code(Index, Smax, a, b, Muster);
            if ((Wert < 255) && (Wert > 0)) {       /* Konturpunkt ?        */
               Buffer1(k, 0) = (unsigned char)a;    /* dann die Koordinaten */
               Buffer1(k, 1) = (unsigned char)b;    /* speichern.           */
               k++;
            }
            else if (Wert == 255) {                 /* Innenpunkt ?         */
               Buffer2(m, 0) = (unsigned char)a;    /* dann die Koordinaten */
               Buffer2(m, 1) = (unsigned char)b;    /* gesondert speichern. */
               m++;
            }
        }
        kmax = k; /* neu ermittelte Anzahl fuer die Buffer-Grenzen uebern.  */
        lmax = m;
  }


  /* Bild zurueckkopieren und dabei wieder in schwarzweiss umkodieren       */
  for (z=0; z<PIC_SIZE; z++)
      Pictures[Index + z] = (Pictures[Index + z] ? WEISS:SCHWARZ);

  farfree(Liste);          /* dynamisch belegten Speicehr wieder frei geben */
  farfree(Buffer1);
  farfree(Buffer2);

  Make_Icon(BildNr);       /* endgueltiges Ergebnis auf dem Bildschirm anz. */
  Show_Pic(BildNr, BildNr+1);

}

/*****************************************************************************
*                                                                            *
* Kodiert die Nachbarpukte eines Pixels und legt das Skelettierungsmuster    *
* fest.                                                                      *
*                                                                            *
*****************************************************************************/

unsigned char Code(long BildOffset, int Spaltenzahl, long z, long s, unsigned char Muster[])
{
  unsigned char Nachbar[8];

  /* Zuerst Einlesen der 8 Nachbarpunkte */
  /* ----------------------------------- */
  /*              7 0 1                  */
  /*              6 P 2                  */
  /*              5 4 3                  */
  /* ----------------------------------- */
  Nachbar[0] = Pictures[BildOffset + (long)Spaltenzahl*(z-1) + (s  )];
  Nachbar[1] = Pictures[BildOffset + (long)Spaltenzahl*(z-1) + (s+1)];
  Nachbar[2] = Pictures[BildOffset + (long)Spaltenzahl*(z  ) + (s+1)];
  Nachbar[3] = Pictures[BildOffset + (long)Spaltenzahl*(z+1) + (s+1)];
  Nachbar[4] = Pictures[BildOffset + (long)Spaltenzahl*(z+1) + (s  )];
  Nachbar[5] = Pictures[BildOffset + (long)Spaltenzahl*(z+1) + (s-1)];
  Nachbar[6] = Pictures[BildOffset + (long)Spaltenzahl*(z  ) + (s-1)];
  Nachbar[7] = Pictures[BildOffset + (long)Spaltenzahl*(z-1) + (s-1)];


  /* Skelettmuster                            */
  /* Die mit p bezeichneten Pixel sind        */
  /* Skelettpunkte und nicht entfernt werden. */
  /* ---------------------------------------- */
  /*           0                   1          */
  /* must1:  1 p 1       must2:  0 p 0        */
  /*           0                   1          */
  /* ---------------------------------------- */
  /*           0 1                 p 0        */
  /* must3:    p 0       must4:    0 1        */
  /* ---------------------------------------- */
  /*           0 p                 1 0        */
  /* must5:    1 0       must6:    0 p        */
  /* ---------------------------------------- */

  Muster[1] = ((Nachbar[0]==0) && (Nachbar[2]==1) && (Nachbar[4]==0) && (Nachbar[6]==1));
  Muster[2] = ((Nachbar[0]==1) && (Nachbar[2]==0) && (Nachbar[4]==1) && (Nachbar[6]==0));
  Muster[3] = ((Nachbar[0]==0) && (Nachbar[1]==1) && (Nachbar[2]==0));
  Muster[4] = ((Nachbar[2]==0) && (Nachbar[3]==1) && (Nachbar[4]==0));
  Muster[5] = ((Nachbar[4]==0) && (Nachbar[5]==1) && (Nachbar[6]==0));
  Muster[6] = ((Nachbar[6]==0) && (Nachbar[7]==1) && (Nachbar[0]==0));

  /* Die 8 Nachbarpunkte werden zur Vereinfachung als 8-Bit-Wert kodiert    */
  return((unsigned char)((Nachbar[7] << 7) + (Nachbar[6] << 6) + (Nachbar[5] << 5)
             + (Nachbar[4] << 4) + (Nachbar[3] << 3) + (Nachbar[2] << 2)
             + (Nachbar[1] << 1) + (Nachbar[0]))); 
}
