/*****************************************************************************
*                                                                            *
* void      Thinning_LW(int BildNr)                                          *
*                                                                            *
******************************************************************************
*                                                                            *
* Skelettierung nach dem Artikel von T.Y. Zhang & C.Y. Suen                  *
* Proceedings Computer Vision and Pattern Recognition                        *
* 1985, Seite 364-367  (Ursprung: Lue/Wang)                                  *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 25.01.1993                                               *
* Letzte Aenderung: 25.01.1993                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Input: BildNr  = Nummer des Ausgangsbildes (1<=BildNr1<=MAX_PICTURE).      *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von: ClearTextWindow(), WriteText(), Geduld(), Read_Int(),          *
*             Make_Icon(), Show_Pic()                                        *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"

#define ERFUELLT   15


void Thinning_LW(int BildNr)
{
  long z, s;              /* Zeilen- und Spaltenzaehler                     */
  long Index, Index1;    /* Anfangsadresse von Bild 1 und Bild 2            */
  long Zeilenzahl, Spaltenzahl;
  int  i, Anzahl;
  int  Bedingung;
  int  Aenderung;
  int  Ende;
  int  P[11];            /* Vektor zur vereinfachten Berechnung der Ueberg. */

  /* Titel nur anzeigen, falls keine bzw. falsche Parameter angegeben       */
  if (BildNr==FALSE) {
     ClearTextWindow(35,18,80,25);
     WriteText(35,18,"* Skelettierung Lue/Wang *");
  }

  if (BildNr == FALSE) {
     gotoxy(35,20);
     BildNr=Read_Int("Bild Nr. ", 1, ':');
  }
  /* Gueltige Bild-Nummer eingegeben? Ist dieses Bild ueberhaupt benutzt?   */
  if ((BildNr<=0) || (BildNr>MAX_PICTURE)) return;
  BildNr--;                           /* die Zaehlung faengt immer bei 0 an */
  if (!Bild[BildNr].Used) return;

  Geduld();

  Index1 = (long) (BildNr) * PIC_SIZE;   /* Anfangsadressen der beiden ...  */

  Spaltenzahl = Bild[BildNr].Spalten;
  Zeilenzahl  = Bild[BildNr].Zeilen;

  /* zuerst den Bildrand auf Schwarz setzen.                                */
  for (s=0; s<Spaltenzahl; s++) {
      Pictures[Index1+s]=SCHWARZ;
      Pictures[Index1+Spaltenzahl*(Zeilenzahl-1)+s]=SCHWARZ;
  }
  for (z=0; z<Zeilenzahl; z++) {
      Pictures[Index1+z*Spaltenzahl]=SCHWARZ;
      Pictures[Index1+z*Spaltenzahl+(Spaltenzahl-1)]=SCHWARZ;
  }

  Ende = FALSE;
  do {
     Aenderung=0;
     /* Zuerst Durchgang von oben nach unten ...                            */
     for (z=1; z<(Zeilenzahl-1); z++) {
         Index = Index1 + z*Spaltenzahl;
         for (s=1; s<(Spaltenzahl-1); s++) {
             if (Pictures[Index + s]) {
                P[2] = Pictures[Index - Spaltenzahl + s    ];
                P[3] = Pictures[Index - Spaltenzahl + s + 1];
                P[4] = Pictures[Index               + s + 1];
                P[5] = Pictures[Index + Spaltenzahl + s + 1];
                P[6] = Pictures[Index + Spaltenzahl + s    ];
                P[7] = Pictures[Index + Spaltenzahl + s - 1];
                P[8] = Pictures[Index               + s - 1];
                P[9] = Pictures[Index - Spaltenzahl + s - 1];
                P[10]=P[2];

                Bedingung=0;
                Anzahl   =0;
                for (i=2; i<=9; i++)
                    if (P[i]) Anzahl++;

                if ((3<=Anzahl) && (Anzahl<=6))     /*  3<=S(P1)<=6  */
                   Bedingung += 1;
                if (!(P[2] && P[4] && P[6]))        /*  P2*P4*P6=0   */
                   Bedingung += 2;
                if (!(P[4] && P[6] && P[8]))        /*  P4*P6*P8=0   */
                   Bedingung += 4;

                Anzahl=0;
                for (i=2; i<10; i++)
                    if (!P[i] && P[i+1]) Anzahl++;  /*  01-Patterns  */

                if (Anzahl==1) Bedingung += 8;
        
                if (Bedingung==ERFUELLT) {
                   Pictures[Index + s] = GRAU;      /* Markierung setzen    */
                   Aenderung++;
                }
             }
         }
     }
     Ende = (Aenderung==0);

     for (z=0; z<PIC_SIZE; z++)
         if (Pictures[Index1 + z]==GRAU) Pictures[Index1 + z]=SCHWARZ;


     Aenderung=0;
     /* ... danach Durchgang von unten nach oben.                           */
     for (z=(Zeilenzahl-2); z>=1; z--) {
         Index = Index1 + z*Spaltenzahl;
         for (s=(Spaltenzahl-2); s>=1; s--) {
             if (Pictures[Index + s]) {
                P[2] = Pictures[Index - Spaltenzahl + s    ];
                P[3] = Pictures[Index - Spaltenzahl + s + 1];
                P[4] = Pictures[Index               + s + 1];
                P[5] = Pictures[Index + Spaltenzahl + s + 1];
                P[6] = Pictures[Index + Spaltenzahl + s    ];
                P[7] = Pictures[Index + Spaltenzahl + s - 1];
                P[8] = Pictures[Index               + s - 1];
                P[9] = Pictures[Index - Spaltenzahl + s - 1];
                P[10]=P[2];

                Bedingung=0;
                Anzahl   =0;
                for (i=2; i<=9; i++)
                    if (P[i]) Anzahl++;

                if ((3<=Anzahl) && (Anzahl<=6))     /*  3<=S(P1)<=6  */
                   Bedingung += 1;
                if (!(P[2] && P[4] && P[8]))        /*  P2*P4*P8=0   */
                   Bedingung += 2;
                if (!(P[2] && P[6] && P[8]))        /*  P4*P6*P8=0   */
                   Bedingung += 4;

                Anzahl=0;
                for (i=2; i<10; i++)
                    if (!P[i] && P[i+1]) Anzahl++;  /*  01-Patterns  */

                if (Anzahl==1) Bedingung += 8;
        
                if (Bedingung==ERFUELLT) {
                   Pictures[Index + s] = GRAU;      /* Markierung setzen    */
                   Aenderung++;
                }
             }
         }
     }
     Ende = (Ende && (Aenderung==0));

     for (z=0; z<PIC_SIZE; z++)       /* alle markierten Pixel nun loeschen */
         if (Pictures[Index1 + z]==GRAU) Pictures[Index1 + z]=SCHWARZ;

  } while (!Ende);

  Make_Icon(BildNr);
  Show_Pic(BildNr, BildNr+1);

}
